package com.huiju.estateagents.center.taUser.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.estateagents.center.taUser.entity.TaTagUser;
import com.huiju.estateagents.center.taUser.entity.TaTags;
import com.huiju.estateagents.center.taUser.entity.TaUser;
import com.huiju.estateagents.center.taUser.entity.TaUserRole;
import com.huiju.estateagents.center.taUser.mapper.TaTagUserMapper;
import com.huiju.estateagents.center.taUser.mapper.TaUserMapper;
import com.huiju.estateagents.center.taUser.mapper.TaUserRoleMapper;
import com.huiju.estateagents.center.taUser.service.ITaUserService;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.MD5Utils;
import com.huiju.estateagents.common.StringUtils;
import com.huiju.estateagents.entity.TaMiniapp;
import com.huiju.estateagents.entity.TaMiniappTemplate;
import com.huiju.estateagents.entity.TaOrg;
import com.huiju.estateagents.entity.TaPerson;
import com.huiju.estateagents.entity.TaPersonBuilding;
import com.huiju.estateagents.mapper.TaMiniappMapper;
import com.huiju.estateagents.mapper.TaMiniappTemplateMapper;
import com.huiju.estateagents.mapper.TaOrgMapper;
import com.huiju.estateagents.mapper.TaPersonBuildingMapper;
import com.huiju.estateagents.mapper.TaPersonMapper;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 用户表  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-09-11
 */
@Service
public class TaUserServiceImpl extends ServiceImpl<TaUserMapper, TaUser> implements ITaUserService {
	
	@Autowired
	private TaUserMapper taUserMapper;

	@Autowired
	private TaOrgMapper taOrgMapper;

	@Autowired
	private TaMiniappMapper taMiniappMapper;

	@Autowired
	private TaMiniappTemplateMapper taMiniappTemplateMapper;
	
	@Autowired
	private TaTagUserMapper taTagUserMapper;
	
	@Autowired
	private TaPersonBuildingMapper taPersonBuildingMapper;
	
	@Autowired
	private TaPersonMapper taPersonMapper;
	
	@Autowired
	private TaUserRoleMapper taUserRoleMapper;
	
	@Override
	public IPage<Map<String,Object>> getPageList(IPage pg) {
		List<Map<String,Object>> taUserList = taUserMapper.getPageList(pg);
		pg.setRecords(taUserList);
		return pg;
	}

	@Override
	public TaUser newUserAndOrg(TaUser taUser) throws Exception {
		if (StringUtils.isEmpty(taUser.getPhone())) {
			throw new Exception("电话号码不能为空");
		}
		if (StringUtils.isEmpty(taUser.getUserName())) {
			throw new Exception("用户名称不能为空");
		}

		if (StringUtils.isEmpty(taUser.getLoginName())) {
			taUser.setLoginName(taUser.getPhone());
		}

		QueryWrapper<TaUser> wrapper = new QueryWrapper<>();
		wrapper.eq("phone", taUser.getPhone()).or().eq("login_Name", taUser.getLoginName());
		List<TaUser> taUserList = this.list(wrapper);
		if (null != taUserList && taUserList.size() > 0) {
			throw new Exception("电话号码或登录名重复");
		}

		// 先插入 org
		TaOrg taOrg = new TaOrg();
		taOrg.setName(taUser.getOrgName());
		taOrg.setWaterMark(taUser.getWaterMark());
		taOrg.setCreateDate(LocalDateTime.now());
		if (taOrgMapper.insert(taOrg) < 1) {
			throw new Exception("生成用户公司信息失败");
		}

		taUser.setOrgId(taOrg.getOrgId());
		taUser.setIsAdmin(true);
		taUser.setLoginPassword(MD5Utils.md5(taUser.getLoginPassword()));
		taUser.setStatus(CommConstant.STATUS_NORMAL);
		taUser.setCreateDate(LocalDateTime.now());
		if (!this.save(taUser)) {
			throw new Exception("生成用户信息失败");
		}

		return taUser;
	}

	@Override
	public TaUser getInfoWithOrg(Integer id) {
		TaUser taUser = this.getById(id);
		if (null == taUser) {
			return taUser;
		}

		TaOrg taOrg = taOrgMapper.selectById(taUser.getOrgId());
		if (null != taOrg) {
			taUser.setOrgName(taOrg.getName());
			taUser.setWaterMark(taOrg.getWaterMark());
		}

		QueryWrapper<TaMiniapp> wrapper = new QueryWrapper<>();
		wrapper.eq("org_id", taOrg.getOrgId());

		TaMiniapp taMiniapp = taMiniappMapper.selectOne(wrapper);
		if (null != taMiniapp) {
			QueryWrapper<TaMiniappTemplate> query = new QueryWrapper<>();
			query.eq("miniapp_id", taMiniapp.getMiniappId());
			taMiniapp.setTpls(taMiniappTemplateMapper.selectList(query));
		}

		taUser.setMiniapp(taMiniapp);
		taUser.setLoginPassword(null);
		return taUser;
	}

	@Override
	public void updateInfoWithOrg(TaUser taUser) throws Exception {
		TaOrg taOrg = taOrgMapper.selectById(taUser.getOrgId());
		if (null != taOrg) {
			taOrg.setName(taUser.getOrgName());
			taOrg.setWaterMark(taUser.getWaterMark());

			if (taOrgMapper.updateById(taOrg) < 1) {
				throw new Exception("更新用户公司信息失败");
			}
		}
		
		taUser.setLoginPassword(null);
		if (taUserMapper.updateById(taUser) < 1) {
			throw new Exception("更新用户信息失败");
		}
	}

	@Override
	public void turnUserOnOrNot(Integer id, boolean isOn) throws Exception {
		TaUser taUser = this.getById(id);
		if (null == taUser) {
			throw new Exception("未找到有效的用户信息");
		}

		Integer status = isOn ? CommConstant.STATUS_NORMAL : CommConstant.STATUS_CLOSED;
		TaUser user = new TaUser();
		user.setStatus(status);
		user.setUserId(id);

		if (!this.updateById(user)) {
			throw new Exception("开启或禁用用户失败");
		}
	}

	@Override
	public void resetPassword(Integer id, String newPass) throws Exception {
		TaUser taUser = this.getById(id);
		if (null == taUser) {
			throw new Exception("未找到有效的用户信息");
		}

		String encryptPass = MD5Utils.md5(MD5Utils.md5(newPass));
		TaUser user = new TaUser();
		user.setLoginPassword(encryptPass);
		user.setUserId(id);

		if (!this.updateById(user)) {
			throw new Exception("重置用户密码失败");
		}
	}
	
	@Override
	public List<TaTags> getTagsList(TaUser e) {
		return taUserMapper.getTagsList(e.getUserId(),e.getOrgId());
	}
	
	@Override
	public TaUser  addAdminUser(TaUser taUser) throws Exception {
		if (StringUtils.isEmpty(taUser.getPhone())) {
			throw new Exception("电话号码不能为空");
		}
		if (StringUtils.isEmpty(taUser.getUserName())) {
			throw new Exception("用户名称不能为空");
		}
		
		if (StringUtils.isEmpty(taUser.getLoginName())) {
			taUser.setLoginName(taUser.getPhone());
		}
		
		QueryWrapper<TaUser> wrapper = new QueryWrapper<>();
		wrapper.eq("phone", taUser.getPhone()).or().eq("login_Name", taUser.getLoginName());
		wrapper.eq("org_id",taUser.getOrgId());
		List<TaUser> taUserList = this.list(wrapper);
		if (null != taUserList && taUserList.size() > 0) {
			throw new Exception("电话号码或登录名重复");
		}
		taUser.setLoginPassword(MD5Utils.md5(MD5Utils.md5("123456")));
		taUser.setStatus(CommConstant.STATUS_NORMAL);
		taUser.setCreateDate(LocalDateTime.now());
		if (taUserMapper.insert(taUser) < 1) {
			throw new Exception("生成用户信息失败");
		}
		
		//保存标签
		List<Integer> tags = taUser.getTaTags();
		tags.forEach(e -> {
			TaTagUser taTagUser = new TaTagUser();
			taTagUser.setUserId(taUser.getUserId());
			taTagUser.setTagId(e);
			taTagUserMapper.insert(taTagUser);
		});
		
		//保存用户角色
		List<Integer> roleIds = taUser.getRoleIds();
		roleIds.forEach(e -> {
			TaUserRole taUserRole = new TaUserRole();
			taUserRole.setUserId(taUser.getUserId());
			taUserRole.setRoleId(e);
			taUserRoleMapper.insert(taUserRole);
		});
		
		//保存项目
		TaPersonBuilding taPersonBuilding = new TaPersonBuilding();
		taPersonBuilding.setBuildingId(taUser.getBuildingId());
		taPersonBuilding.setUserId(taUser.getUserId());
		taPersonBuilding.setCreateDate(LocalDateTime.now());
		taPersonBuildingMapper.insert(taPersonBuilding);
		return taUser;
	}
	
	@Override
	public TaUser updateAdminUser(TaUser taUser) throws Exception {
		TaUser taUserBefore = taUserMapper.selectById(taUser.getUserId());
		if (StringUtils.isEmpty(taUser.getPhone())) {
			throw new Exception("电话号码不能为空");
		}
		if (StringUtils.isEmpty(taUser.getUserName())) {
			throw new Exception("用户名称不能为空");
		}
		if(StringUtils.isEmpty(taUser.getLoginName()) && taUserBefore.getLoginName().equals(taUserBefore.getPhone())){
			taUser.setLoginName(taUser.getPhone());
		}

		QueryWrapper<TaUser> wrapper = new QueryWrapper<>();
		wrapper.nested(q -> q.eq("phone", taUser.getPhone()).or().eq("login_Name", taUser.getLoginName()));
		wrapper.eq("org_id",taUser.getOrgId());
		wrapper.ne("user_id",taUser.getUserId());
		List<TaUser> taUserList = this.list(wrapper);
		if (null != taUserList && taUserList.size() > 0) {
			throw new Exception("电话号码或登录名重复");
		}
		if (taUserMapper.updateById(taUser) < 1){
			throw new Exception("更新用户信息失败");
		}
		QueryWrapper<TaTagUser> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("user_id",taUser.getUserId());
		taTagUserMapper.delete(queryWrapper);
		//保存标签
		List<Integer> tags = taUser.getTaTags();
		tags.forEach(e -> {
			TaTagUser taTagUser = new TaTagUser();
			taTagUser.setUserId(taUser.getUserId());
			taTagUser.setTagId(e);
			taTagUserMapper.insert(taTagUser);
		});
		
		QueryWrapper<TaUserRole> taUserRoleQueryWrapper = new QueryWrapper<>();
		taUserRoleQueryWrapper.eq("user_id",taUser.getUserId());
		taUserRoleMapper.delete(taUserRoleQueryWrapper);
		//保存用户角色
		List<Integer> roleIds = taUser.getRoleIds();
		roleIds.forEach(e -> {
			TaUserRole taUserRole = new TaUserRole();
			taUserRole.setUserId(taUser.getUserId());
			taUserRole.setRoleId(e);
			taUserRoleMapper.insert(taUserRole);
		});
		
		QueryWrapper<TaPersonBuilding> taPersonBuildingQueryWrapper = new QueryWrapper<>();
		taPersonBuildingQueryWrapper.eq("user_id",taUser.getUserId());
		taPersonBuildingMapper.delete(taPersonBuildingQueryWrapper);
		//保存项目
		TaPersonBuilding taPersonBuilding = new TaPersonBuilding();
		taPersonBuilding.setBuildingId(taUser.getBuildingId());
		taPersonBuilding.setUserId(taUser.getUserId());
		taPersonBuilding.setCreateDate(LocalDateTime.now());
		taPersonBuildingMapper.insert(taPersonBuilding);
		
		//更新person
		TaPerson taPerson = new TaPerson();
		taPerson.setAvatarurl(taUser.getAvatar());
		taPerson.setCompany(taUser.getOrgName());
		taPerson.setTel(taUser.getPhone());
		taPerson.setName(taUser.getUserName());
		taPerson.setAddress(taUser.getAddress());
		taPerson.setIdNo(taUser.getIdNo());
		taPerson.setDepartment(taUser.getDepartment());
		taPerson.setPost(taUser.getPosition());
		taPerson.setPhoto(taUser.getPhoto());
		QueryWrapper<TaPerson> taPersonQueryWrapper = new QueryWrapper<>();
		taPersonQueryWrapper.eq("user_id",taUser.getUserId());
		taPersonMapper.update(taPerson,taPersonQueryWrapper);
		
		return taUser;
	}
}
