package com.huiju.estateagents.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.entity.HelpActivity;
import com.huiju.estateagents.entity.TaBuildingDynamic;
import com.huiju.estateagents.entity.TaChannel;
import com.huiju.estateagents.service.IHelpActivityService;
import com.huiju.estateagents.service.ITaBuildingService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.apache.ibatis.annotations.Param;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.xml.crypto.Data;

/**
 * <p>
    * 助力活动  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2019-10-17
 */
@RestController
@RequestMapping("/api")
@Api(value = "活动助力", tags = "活动助力")
public class HelpActivityController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(HelpActivityController.class);

    @Autowired
    public IHelpActivityService taHelpActivityService;

    @Autowired
    private ITaBuildingService iTaBuildingService;


    /**
     * 活动助力列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @ApiOperation(value = "活动助力列表", notes = "活动助力列表")
    @ApiImplicitParams({
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageNum", paramType = "query",value = "第几页"),
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageSize", paramType = "query",value = "一页多少行"),
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "cityId", paramType = "query",value = "城市"),
            @ApiImplicitParam(dataTypeClass = String.class, name = "buildingId", paramType = "query",value = "项目"),
            @ApiImplicitParam(dataTypeClass = String.class, name = "title", paramType = "query",value = "标题"),
            @ApiImplicitParam(dataTypeClass = Data.class, name = "startDate", paramType = "query",value = "活动开始时间"),
            @ApiImplicitParam(dataTypeClass = Data.class, name = "endDate", paramType = "query",value = "活动结束时间"),
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "status", paramType = "query",value = "活动状态")
    })
    @RequestMapping(value="admin/helpActivity/list",method= RequestMethod.GET)
    public ResponseBean helpActivityList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                         @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                         @RequestParam(value ="cityId",required = false) Integer cityId,
                                         @RequestParam(value ="buildingId",required = false) String buildingId,
                                         @RequestParam(value ="title",required = false) String title,
                                         @RequestParam(value ="startDate",required = false) Data startDate,
                                         @RequestParam(value ="endDate",required = false) Data endDate,
                                         @RequestParam(value ="status",required = false) Integer status,
                                         HttpServletRequest request){
        Integer orgId = getOrgId(request);
        ResponseBean  helpActivityList= taHelpActivityService.helpActivityList(pageNum,pageSize,cityId,buildingId,title,startDate,endDate,status,orgId);
        return helpActivityList;
    }

    @ApiOperation(value = "活动助力新增", notes = "活动助力新增")
    @ApiImplicitParams({
            @ApiImplicitParam(dataType = "HelpActivity", name = "parameter", paramType = "body",value = "当前对象")
    })
    @RequestMapping(value="admin/helpActivity/add",method= RequestMethod.POST)
    public ResponseBean helpActivityAdd(@RequestBody String parameter,HttpServletRequest request){
        HelpActivity helpActivity = JSONObject.parseObject(parameter, HelpActivity.class);
        ResponseBean responseBean = new ResponseBean();
        Integer orgId = getOrgId(request);
        Integer cityId = iTaBuildingService.getCityById(helpActivity.getBuildingId());
        helpActivity.setCityId(cityId);
        helpActivity.setWeight(0);
        helpActivity.setIsMain(false);
        taHelpActivityService.helpActivityAdd(helpActivity,orgId);
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/helpActivity/{id}", method= RequestMethod.DELETE)
    public ResponseBean helpActivityDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(taHelpActivityService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("helpActivityDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }


    @ApiOperation(value = "活动助力修改", notes = "活动助力修改")
    @ApiImplicitParams({
            @ApiImplicitParam(dataTypeClass = HelpActivity.class, name = "helpActivity", paramType = "body",value = "当前对象"),
    })
    @RequestMapping(value="admin/helpActivity/update",method= RequestMethod.PUT)
    public ResponseBean helpActivityUpdate(@RequestBody String parameter){
        ResponseBean responseBean = new ResponseBean();
        HelpActivity helpActivity = JSONObject.parseObject(parameter, HelpActivity.class);
        helpActivity.setHelpActivityId(helpActivity.getHelpActivityId());
        taHelpActivityService.helpActivityUpdate(helpActivity);
        return responseBean;
    }

    @ApiOperation(value = "根据id查询活动助力详情", notes = "根据id查询活动助力详情")
    @ApiImplicitParams({
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "helpActivityId", paramType = "query",value = "活动状态")
    })
    @RequestMapping(value="admin/helpActivity/details",method= RequestMethod.GET)
    public ResponseBean helpActivityGet( @RequestParam(value ="helpActivityId",required = false) Integer helpActivityId){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(taHelpActivityService.getById(helpActivityId));
        }catch (Exception e){
            e.printStackTrace();
            logger.error("helpActivityDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
    @ApiOperation(value = "置顶和推首页", notes = "置顶和推首页")
    @ApiImplicitParams({
            @ApiImplicitParam(dataTypeClass = String.class, name = "parameter", paramType = "body",value = "status:0置顶操作，1推首页操作,2结束活动操作,helpActivityId:助力ID")
    })
    @RequestMapping(value="admin/top/update",method= RequestMethod.POST)
    public ResponseBean top( @RequestBody String parameter){
        ResponseBean responseBean = new ResponseBean();
        JSONObject object= JSONObject.parseObject(parameter);
        // 根据状态值判断是置顶还是推首页的操作
        String  status= object.getString("status");
        String  helpActivityId= object.getString("helpActivityId");
        HelpActivity helpActivity = taHelpActivityService.getById(helpActivityId);
        // 置顶操作
        if (status.equals("0")) {
            helpActivity.setWeight(Integer.valueOf(helpActivity.getWeight().equals(0)? 1:0));
        }
        // 推首页
        if (status.equals("1")){
            helpActivity.setIsMain(true==helpActivity.getIsMain()?false:true);
        }
        // 结束活动操作
        if (status.equals("2")){
            helpActivity.setStatus(2);
        }
        Boolean isA = taHelpActivityService.updateById(helpActivity);
        responseBean.addSuccess(isA);
        return responseBean;
    }
    @ApiOperation(value = "助力记录", notes = "助力记录")
    @ApiImplicitParams({
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageNum", paramType = "query",value = "第几页"),
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageSize", paramType = "query",value = "一页多少行"),
            @ApiImplicitParam(dataTypeClass = String.class, name = "helpActivityId", paramType = "query",value = "助力ID"),
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "status", paramType = "query",value = "status: 0助力成功，1进行中，2助力失败")
    })
    @RequestMapping(value="admin/helpRecord",method= RequestMethod.GET)
    public ResponseBean helpRecord(@RequestParam(value ="helpActivityId",required = false) Integer helpActivityId,
                                   @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                   @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                   @RequestParam(value ="status",required = false) String status){
        ResponseBean helpRecord = taHelpActivityService.helpRecord(helpActivityId,status, pageNum, pageSize);
        return helpRecord;
    }

    @ApiOperation(value = "助力人详情", notes = "助力人详情")
    @ApiImplicitParams({
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageNum", paramType = "query",value = "第几页"),
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageSize", paramType = "query",value = "一页多少行"),
            @ApiImplicitParam(dataTypeClass = String.class, name = "helpRecordInitiateId", paramType = "query",value = "发起助力ID"),
    })
    @RequestMapping(value="admin/taHelpRecord/details",method= RequestMethod.GET)
    public ResponseBean taHelpRecordDetails(@RequestParam(value ="helpRecordInitiateId",required = false) Integer helpRecordInitiateId,
                                   @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                   @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize){
        ResponseBean helpRecord = taHelpActivityService.taHelpRecordDetails(helpRecordInitiateId, pageNum, pageSize);
        return helpRecord;
    }

    @ApiOperation(value = "核销", notes = "核销")
    @ApiImplicitParams({
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageNum", paramType = "query",value = "第几页"),
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "pageSize", paramType = "query",value = "一页多少行"),
            @ApiImplicitParam(dataTypeClass = String.class, name = "helpRecordInitiateId", paramType = "query",value = "发起助力ID"),
            @ApiImplicitParam(dataTypeClass = Integer.class, name = "verifyCode", paramType = "query",value = "核销码"),
    })
    @RequestMapping(value="admin/helpInitiateRecordVerify/",method= RequestMethod.POST)
    public ResponseBean helpInitiateRecordVerify(@RequestParam(value ="helpRecordInitiateId",required = false) Integer helpRecordInitiateId,
                                                 @RequestParam(value ="verifyCode",required = false) Integer verifyCode){
        ResponseBean helpRecord = taHelpActivityService.helpInitiateRecordVerify(helpRecordInitiateId,verifyCode);
        return helpRecord;
    }
}
