package com.huiju.estateagents.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.entity.HelpActivity;
import com.huiju.estateagents.entity.HelpInitiateRecord;
import com.huiju.estateagents.entity.HelpRecord;
import com.huiju.estateagents.mapper.HelpActivityMapper;
import com.huiju.estateagents.mapper.HelpInitiateRecordMapper;
import com.huiju.estateagents.mapper.HelpRecordMapper;
import com.huiju.estateagents.service.IHelpActivityService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.xml.crypto.Data;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;

/**
 * <p>
 * 助力活动  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-10-17
 */
@Service
public class HelpActivityServiceImpl extends ServiceImpl<HelpActivityMapper, HelpActivity> implements IHelpActivityService {

    @Autowired
    HelpActivityMapper taHelpActivityMapper;

    @Autowired
    HelpInitiateRecordMapper helpInitiateRecordMapper;

    @Autowired
    HelpRecordMapper helpRecordMapper;

    @Override
    public ResponseBean helpActivityList(Integer pageNum, Integer pageSize, Integer cityId, String buildingId, String title, Data startDate, Data endDate, Integer status, Integer orgId) {
        ResponseBean responseBean = new ResponseBean<>();
        IPage<HelpActivity> pg = new Page<>(pageNum, pageSize);
        IPage<HelpActivity> result = taHelpActivityMapper.helpActivityListPage(pg, cityId, buildingId, title, startDate, endDate, status, orgId);
        responseBean.addSuccess(result);
        return responseBean;
    }

    @Override
    public void helpActivityAdd(HelpActivity helpActivity, Integer orgId) {
        helpActivity.setOrgId(orgId)
                .setCreateDate(LocalDateTime.now());
        taHelpActivityMapper.insert(helpActivity);
    }

    @Override
    public void helpActivityUpdate(HelpActivity helpActivity) {
        taHelpActivityMapper.updateById(helpActivity);
    }

    @Override
    public ResponseBean helpRecord(Integer helpActivityId, String status,Integer pageNum,Integer pageSize) {
        ResponseBean responseBean = new ResponseBean<>();
        //  status: helpSucceed助力成功，helpUnderway进行中，helpUnfinished助力失败

        //查询当前助力成功所需的人数限制（persion_num）
        HelpActivity helpActivity = taHelpActivityMapper.selectById(helpActivityId);
        // 得到活动限制助力的总数
        int persionNumCount = helpActivity.getPersionNum();
        IPage<HelpInitiateRecord> pg = new Page<>(pageNum, pageSize);
        IPage<HelpInitiateRecord> list = helpInitiateRecordMapper.selectPageList(pg,helpActivityId,status);

        List<HelpInitiateRecord>  helpInitiateRecordList= list.getRecords();
        for (HelpInitiateRecord helpInitiateRecord : helpInitiateRecordList) {
            helpInitiateRecord.setPersionNumCount(persionNumCount);
            // 统计助力成功的总数(发起人得到助力数满了即可)
            QueryWrapper<HelpRecord> helpRecorddQueryWrapper = new QueryWrapper<>();
            helpRecorddQueryWrapper.eq("help_record_initiate_id", helpInitiateRecord.getHelpRecordInitiateId());
            // 拿到每个发起人，得到助力的总数
            int helpRecordCount = helpRecordMapper.selectCount(helpRecorddQueryWrapper);
            // 助力成功
            if ("helpSucceed".equals(status)) {
                if (persionNumCount == helpRecordCount) {
                    helpInitiateRecord.setHelpCount(helpRecordCount);
                }
            }
            // 助力进行中（活动进行中，未结束）
            if ("helpUnderway".equals(status) && "1".equals(helpActivity.getStatus())) {
                if (persionNumCount > helpRecordCount) {
                    helpInitiateRecord.setHelpCount(helpRecordCount);
                }
            }
            // 活动助力失败
            // 活动进行中（活动已结束）
            if ("helpUnfinished".equals(status) && "2".equals(helpActivity.getStatus())) {
                if (persionNumCount > helpRecordCount) {
                    helpInitiateRecord.setHelpCount(helpRecordCount);
                }

            }
        }
        HashMap<String, Object> map = new HashMap<>();
        map.put("list", helpInitiateRecordList);
        map.put("total", list.getTotal());
        map.put("pageNum", list.getCurrent());
        map.put("pageSize", list.getSize());
        return ResponseBean.success(map);
    }

    @Override
    public ResponseBean taHelpRecordDetails(Integer helpRecordInitiateId, Integer pageNum, Integer pageSize) {
        ResponseBean responseBean = new ResponseBean<>();
        IPage<HelpInitiateRecord> pg = new Page<>(pageNum, pageSize);
        IPage<HelpRecord> helpRecordList= helpRecordMapper.taHelpRecordDetails(pg,helpRecordInitiateId);
        responseBean.addSuccess(helpRecordList);
        return responseBean;
    }

    @Override
    public ResponseBean helpInitiateRecordVerify(Integer helpRecordInitiateId, Integer verifyCode) {
        ResponseBean responseBean = new ResponseBean<>();
        HelpInitiateRecord helpInitiateRecord = helpInitiateRecordMapper.selectById(helpRecordInitiateId);
        // 判断是否已核销
        if ("1".equals(helpInitiateRecord.getVerificationStatus())){
            responseBean.addSuccess("已核销");
            return responseBean;
        }

        if (helpInitiateRecord.getVerification().equals(verifyCode)){
            responseBean.addSuccess("成功");
        }else {
            responseBean.addError("核销码错误");
        }
        return responseBean;
    }
}